﻿###############################################################################
#                                                                             #
#                     Change Mailbox Password for Office 365                  #
#                                  V3.9.8.6                                   #
#                       (C)2006-2021 KoXo Developpement                       #
#                                                                             #
###############################################################################
function log_message
{
Param($Header,$Message)	
$LogDate=Get-Date
$LogInfo="[$($LogDate)]: [$($Header)] $($Message)"
$LogInfo | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function log_total_errors_count
{
$S="TotalErrorsCount=$($global:total_errors_count)"
$S | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function log_errors_count
{
Param($ID)
$S="ErrorsCount[$($ID)]=$($global:errors_count)"
$S | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function check_version
{
$S=Get-Host
if ($S.Version.Major -lt "2")
   {
   $S="Powershell's version must be greater or equal to 2.0 !"
   log_message "Version" $S
   $global:total_errors_count++
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host
   exit
   }
}
###############################################################################
function Banner
{
Param($MenuLines)
$MarginWidth=2
$MarginChar=" "
$MaxWidth=0;
$BorderTextColor="Magenta"
ForEach ($Line In $MenuLines)
        {
        If ($Line.Length -gt $MaxWidth) {$MaxWidth=$Line.Length}
        }
If ($MaxWidth -gt 0)
   {
   Write-Host "╔" -NoNewline -ForegroundColor $BorderTextColor
   For ($i=0; $i -lt $MaxWidth+2*$MarginWidth; $i++)
       {
       Write-Host "═" -NoNewline -ForegroundColor $BorderTextColor
       }
   Write-Host "╗" -ForegroundColor $BorderTextColor
   ForEach ($Line In $MenuLines)
           {
           Write-Host "║" -NoNewline -ForegroundColor $BorderTextColor
           Write-Host ("{0}{1}{2}" -f $MarginChar.PadLeft($MarginWidth),$Line.PadRight($MaxWidth),$MarginChar.PadLeft($MarginWidth)) -NoNewline -ForegroundColor White
           Write-Host "║" -ForegroundColor $BorderTextColor
           }
   Write-Host "╚" -NoNewline -ForegroundColor $BorderTextColor
   For ($i=0; $i -lt $MaxWidth+2*$MarginWidth; $i++)
       {
       Write-Host "═" -NoNewline -ForegroundColor $BorderTextColor
       }
   Write-Host "╝" -ForegroundColor $BorderTextColor
   }
}
###############################################################################
function welcome
{
$BannerText="Changing user's password for Office 365 $($Version)",
            "(C) 2006-2021 KoXo Developpement"
Banner $BannerText
}
###############################################################################
## Install MSOnline Cmdlets 
function Install-MSOnlineCmdlets()
{
$Res=$false
$DotNET35Path="HKLM:\SOFTWARE\Microsoft\NET Framework Setup\NDP\v3.5"
$DotNet35Present=((Test-Path $DotNET35Path) -and ((Get-ItemProperty $DotNET35Path).Install -eq "1"))
if ($DotNet35Present)
   {
   # Trying to import module
   if (-not(Get-Module -Name "MSOnline")){Get-Module -ListAvailable | Where-Object {$_.name -eq "MSOnline"} | Import-Module}
   # Module is not present
   if (-not(Get-Module -Name "MSOnline"))
      {
      Write-Host "MSOnline Powershell module is not present !" -Foregroundcolor "Red"
      $wc=New-Object System.Net.WebClient
      # Sign-in Assistant
      $SigninAssistantPath="HKLM:\SOFTWARE\Microsoft\MSOIdentityCRL"
      $SigninAssistantPresent=((Test-Path $SigninAssistantPath) -and ((Get-ItemProperty $SigninAssistantPath).TargetDir -ne $null))
      $MyCulture="0x{0:X}" -f(Get-Culture).LCID
      if (!$SigninAssistantPresent)
         {
         if ((Get-WmiObject Win32_OperatingSystem).OSArchitecture.Contains("64"))
         	  # V7.250.4556.0
         	  {$Source="http://www.koxo.net/fichiers/msonline/msoidcli_64.msi"} ## {$Source="http://download.microsoft.com/download/A/7/5/A75FE163-6C03-4E44-8E81-70905727D8E6/msoidcli_64.msi"}
            else
            # V7.250.4556.0
            {$Source="http://www.koxo.net/fichiers/msonline/msoidcli_32.msi"} ## {$Source="http://download.microsoft.com/download/A/7/5/A75FE163-6C03-4E44-8E81-70905727D8E6/msoidcli_32.msi"}
         $Destination=$env:TEMP+"\_SigninAssistant_.msi"
         Write-Host "Trying to download ""Microsoft Online Sign-in Assistant"""
         Write-Host "Url=$($Source)"
         Write-Host "Please, wait ..." -ForegroundColor "Cyan"
         $wc.DownloadFile($Source,$Destination)
         $Install=Start-Process $Destination -Wait -PassThru
         if ($Install.ExitCode -eq 0)
            {
            $S="Microsoft Online Sign-in Assistant installation was OK"
            Write-Host $S -ForegroundColor "Green"
            Log_Message "OK" $S
            }
            else
            {
            $S="Installation failed for Microsoft Online Sign-in Assistant, code : $($Install.ExitCode)"
            Write-Host $S -ForegroundColor "Red"
            Log_Message "ERROR" $S
            } 
         $FileExists=Test-Path $Destination
         If ($FileExists) {Get-Item $Destination | Remove-Item} 
         }
      # MSOnline module
      $MSOnlinePath="HKLM:\SOFTWARE\Microsoft\MSOnlinePowerShell"
      $MSOnlinePresent=((Test-Path $MSOnlinePath) -and ((Get-ItemProperty $MSOnlinePath).InstallPath -ne $null))
      if (!$MSOnlinePresent)
         {
         if ((Get-WmiObject Win32_OperatingSystem).OSArchitecture.Contains("64"))
            {
            # 64 bits
            if ([int](Get-WmiObject Win32_OperatingSystem).Version.Split(".")[0] -lt 6)
               {
               # Windows 2003
               $Source="http://www.koxo.net/fichiers/msonline/AdministrationConfig-fr-x64-patched.msi"
               }
               else
               {
               # Windows 2008 and above
               # V1.1.166.0
               $Source="http://www.koxo.net/fichiers/msonline/AdministrationConfig_3-x64.msi" ## $Source="http://download.connect.microsoft.com/pr/AdministrationConfig_3.msi?t=3edbe1ec-9076-441a-ba1d-6a8ae147d3ea&e=1497200079&h=81bba22aebadb1efceb708c41c45fa85"
               }
            }
            else
            {
            # 32 bits
            if ([int](Get-WmiObject Win32_OperatingSystem).Version.Split(".")[0] -lt 6)
               {
               # Windows 2003
               # V1.0
               $Source="http://www.koxo.net/fichiers/msonline/AdministrationConfig-fr-x86-patched.msi"
               }
               else
               {
               # Windows 2008 and above
               # V1.0
               $Source="http://www.koxo.net/fichiers/msonline/AdministrationConfig-fr-x86.msi" ## $Source="http://go.microsoft.com/fwlink/?linkid=236298" ## $Source=$Source+"&clcid="+$MyCulture
               }
            }
         $Destination=$env:TEMP+"\_MSOnline_.msi"
         write-host "Trying to download ""Microsoft Online Powershell Module"""
         Write-Host "Url=$($Source)"
         Write-Host "Please, wait ..." -ForegroundColor "Cyan"
         $wc.DownloadFile($Source,$Destination)
         $Install=Start-Process $Destination -Wait -PassThru
         if ($Install.ExitCode -eq 0)
            {
            $S="Microsoft Online Powershell Module installation was OK"
            Write-Host $S -ForegroundColor "Green"
            Log_Message "OK" $S
            }
            else
            {
            $S="Installation failed for Microsoft Online Powershell Module, code : $($Install.ExitCode)"
            Write-Host $S -ForegroundColor "Red"
            Log_Message "ERROR" $S
            }
         $FileExists=Test-Path $Destination
         if ($FileExists) {Get-Item $Destination | Remove-Item} 
         }
      # Import MSOnline Module
      $Mod=Import-Module "MSOnline" -PassThru
      if ([string]::IsNullOrEmpty($Mod)) 
         {
         $S="MSOnline Powershell module is still not present !"
         Write-Host $S -Foregroundcolor "Red"
         Log_Message "ERROR" $S
         Write-Host "Press any key ..."
         Read-Host
         }
         else
         {
         $S="MSOnline V{0} Powershell module is now present and loaded !"
         Write-Host ($S -f $Mod.Version.ToString()) -Foregroundcolor "Green"
         Log_Message "OK" $S
         $Res=$true
         }
      }
      else
      {
      $Mod=Get-Module MSOnline
      $S="MSOnline V{0} Powershell module is present"
      Write-Host ($S -f $Mod.Version.ToString()) -Foregroundcolor "Green"
      $Res=$true
      }
  }
  else
  {
   $S="Please, install Framework .NET V3.5 first !"
   Write-Host $S -ForegroundColor "Red"
   Log_Message "ERROR" $S
   Write-Host "Press any key ..."
   Read-Host
  }
return $Res
}
###############################################################################
function connect_msol
{
Param([string] $MyName,$MyPassword)
$error.Clear()
# Set credential
if ( ([String]::IsNullorEmpty($MyPassword)) -or ([String]::IsNullorEmpty($MyName))  )
   {
   $Cred=Get-Credential
   }
   else
   {
   $SecuredPassword=ConvertTo-SecureString $MyPassword -AsPlainText -Force
   $Cred=New-Object System.Management.Automation.PSCredential $MyName, $SecuredPassword
   }
%{Connect-MsolService -Credential $Cred}>$Result   
if ([String]::IsNullorEmpty($error[0]))
   {
   $S="Connectionion was successful to Microsoft Online Services"
	 log_message "OK" $S
   } 
   else 
   {
   $global:total_errors_count++
   $S="Connection was unsuccessful to Microsoft Online Services || $($error[0])"
   log_message "ERROR" $S
   # Display error
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host   
   }   
}
###############################################################################
function change_user_password
{
Param($UserEmail,$UserPassword,$UserDisplayName,$UserMustChangePassword,$StrongPasswordRequired,$PasswordNeverExpires)
$error.Clear()
if ($UserMustChangePassword -isnot[Boolean]) {$UserMustChangePassword=$false}
if ($StrongPasswordRequired -isnot[Boolean]) {$StrongPasswordRequired=$false}
if ($PasswordNeverExpires -isnot[Boolean]) {$PasswordNeverExpires=$true}
$error.Clear()
Set-MsolUser -UserPrincipalName $UserEmail -StrongPasswordRequired $StrongPasswordRequired -PasswordNeverExpires $PasswordNeverExpires -ErrorAction SilentlyContinue
if ([String]::IsNullorEmpty($error[0]))
	{
    $S="Successfully configure password policy for ""$($UserDisplayName)"" ""$($UserEmail)"""
	log_message "OK" $S
	}
	else
	{
	$global:errors_count++
    $S="Configuring password policy for ""$($UserDisplayName)"" ""$($UserEmail)"". Error = $($error[0])"
    log_message "ERROR" $S
    }
Set-MsolUserPassword -UserPrincipalName $UserEmail -NewPassword $UserPassword -ForceChangePassword $UserMustChangePassword -ErrorAction SilentlyContinue
if ([String]::IsNullorEmpty($error[0]))
	{
    $S="Successfully change password for ""$($UserDisplayName)"" ""$($UserEmail)"""
	log_message "OK" $S
	}
	else
	{
	$global:errors_count++
    $S="Changing password for ""$($UserDisplayName)"" ""$($UserEmail)"". Error = $($error[0])"
    log_message "ERROR" $S
    }
}
#
###############################################################################
function display_user
{
Param ($UserDisplayName,$UserEmail)
Write-Host "User: """ -nonewline
Write-Host $UserDisplayName -foregroundcolor "Green" -nonewline
Write-Host """ Email: """ -nonewline
Write-Host $UserEmail -foregroundcolor "Green" -nonewline
Write-host """"	
}
###############################################################################
function UserMSolExist
{
Param([string] $email)
$User=Get-MSolUser -UserPrincipalName $email -ErrorAction SilentlyContinue
return (![string]::IsNullOrEmpty($User))
}
###############################################################################
function exec
{
Param($UserEmail,$UserPassword,$UserDisplayName,$UserMustChangePassword,$StrongPasswordRequired,$PasswordNeverExpires)	
$UserDisplayName=$UserDisplayName.Trim()
log_message "OFFICE365" "$($UserDisplayName) ($($UserEmail))"
if (UserMSolExist($UserEmail))
   {
   display_user	$UserDisplayName $UserEmail
   change_user_password $UserEmail $UserPassword $UserDisplayName $UserMustChangePassword $StrongPasswordRequired $PasswordNeverExpires
   }
   else
   {
   $errors_count++
   $S="User ""$($UserEmail)"" does not exists"
   write-host $S -foregroundcolor "red"
   log_message "ERROR" $S
   }
log_errors_count($UserEmail)   
$global:total_errors_count=$global:total_errors_count+$global:errors_count
}
###############################################################################
#                                                                             #
#                                    Main                                     #
#                                                                             #
###############################################################################
#
$global:Version="V3.9.8.6"
#
# Office 365 params
#
$Office365LoginName="%OFFICE365_ADMIN%"
$Office365LoginPassword=[System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("%OFFICE365_BASE64_PASSWORD%"))
#
# Proxy parameters
#
$ProxyEnabled=$#BOOL{%PROXY_ENABLED%}
$ProxyUser="%PROXY_USER%"     
$ProxyPassword=[System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("%PROXY_BASE64_PASSWORD%"))
#
#
$LogFile=$MyInvocation.MyCommand.Definition 
$LogFile=$LogFile -replace(".ps1",".log")
$global:total_errors_count=0
welcome
check_version
if (Install-MSOnlineCmdlets)
   {
   connect_msol $Office365LoginName $Office365LoginPassword
   if ($global:total_errors_count -eq 0)
      {
      $UserMustChangePassword=$#BOOL{%OFFICE365_RESET_PASSWORD_ON_NEXT_LOGON%}
      # You must define these two parameters in KoXo Administrator (Global or Local Variables)
      $StrongPasswordRequired=$#BOOL{%OFFICE365_STRONG_PASSWORD_REQUIRED%}
      $PasswordNeverExpires=$#BOOL{%OFFICE365_PASSWORD_NEVER_EXPIRES%}
      ### BEGIN_REPETITION ###
      $global:errors_count=0
      $UserPassword=[System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("%USER_BASE64_PASSWORD%"))
      $UserDisplayName="%USER_TITLE% %USER_FIRST_NAME% %USER_LAST_NAME%"
      $UserEmail="%USER_EMAIL%"
      exec $UserEmail $UserPassword $UserDisplayName $UserMustChangePassword $StrongPasswordRequired $PasswordNeverExpires
      ### END_REPETITION ###
      }   
   }
   else
   {
   $global:total_errors_count++	
   }	
log_total_errors_count
